/***************************************************************************
                          animation.h  -  description
                             -------------------
    begin                : Mit Aug 21 2002
    copyright            : (C) 2002 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ANIMATION_H
#define ANIMATION_H

#include "tpbase.h"
#include "soundhandle.h"
#include "cdp.h"
#include "cachablepixmaparray.h"
#include <qcanvas.h>
#include <qstring.h>

/** This class is used to store all the information for a (generalized) animation.
	To be used with AnimatedSprite. Contains sound-support and a function to read
	in the information from a QDomElement.
  *@author Thomas Friedrichsmeier
  */

class QDomElement;

class Animation : public TPBase  {
public: 
/** Constructor.
	@param description a pointer to the QDomElement containing the definition for this animation
	@param options options for reading this animation (see TPBase::AnimConfigOptions)
	@param soundoptions if a sound may be associated with this animation, options for reading this sound (see TPBase::SoundConfigOptions)
	@param suffix_a if the Animation may have two orientations, suffix for orientation-specific attributes
	@param suffix_b see above. If suffix_a == suffix_b, only a single animation/orientation will be used
	@return a pointer to this animation */
	Animation(const QDomElement *description, const int options = 0, const int soundoptions = 0, QString suffix_a = "", QString suffix_b = "");
/** Destructor. */
	~Animation();
/** @return true, if this Animation has more than one frame (false for static images) */
	bool multiFrame () { return (frames > 1); }
/** @return the frame-period (in msecs) for this animation */
	int framePeriod () { return (frame_period); }
private:
/** called from the constructor, basically just passing all arguments */
	void readAnimation (const QDomElement *descr, QString & sufa, QString & sufb, const int opts, const int soundopts);

	friend class AnimatedSprite;

/** SoundHandle for this animation */
	SoundHandle sound;
/** Number of repetitions for this animation */
	int repetitions;
/** Whether this animation is continuous */
	bool continuous;
/** Whether this animation is oscillating */
	bool oscillating;
/** Pointer(s) to the QCanvasPixmapArrays used internally */
	CachablePixmapArray * seq[2];
/** Number of frames to be used in this animation */
	int frames;
/** Frame-period in msecs */
	int frame_period;
};

#endif
