/***************************************************************************
                          level.h  -  description
                             -------------------
    begin                : Mon May 21 2001
    copyright            : (C) 2001, 2002 by Thomas Friedrichsmeier
    email                : Thomas.Friedrichsmeier@ruhr-uni-bochum.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef LEVEL_H
#define LEVEL_H

#include "screen.h"

/**
  *@author Thomas Friedrichsmeier
  */

class CenterOfGravity;
class Platform;
class HotArea;
class Teleporter;

class Level:public Screen {
  public:
	Level(const QString & file);
	~Level();
/** The id of the most expensive fuel platform (-1 if no fuel available (should never happen!)) */
	int exp_fuel_platform;

friend class Teleporter;
friend class CenterOfGravity;
friend class Platform;
friend class Event;
friend class Timer;
friend class MovingObject;

	struct Promise {
/** identification string of the promise */
		QString ident;
/** Type of promise (0 is teleporter, 1 is platform, 2 is center of gravity) */		
		int type;
/** Pointer to teleporter (if any) */
		Teleporter *tp_point;
/** Pointer to platform (if any) */
		Platform *plat_point;		
/** Pointer to CoG (if any) */		
		CenterOfGravity *cog_point;
/** Pointer to Object */
		MovingObject *obj_point;
	} *promise_pointer;

/** Normalized base_gravity (not taking into account recalculation period) */
	float n_base_gravity;
/** Normalized horiz_gravity (not taking into account recalculation period) */
	float n_horiz_gravity;	

/** basic downwards gravity */
	float base_gravity;
/** horizontal "gravity" */
	float horiz_gravity;

/** Add a directory prefix to a filename if appropriate (i.e. not an absolute path) */
	unsigned int num_platforms;
/** Number of Centers of Gravity (grav_center) */
	unsigned int num_grav_centers;
/** Number of teleporters. Should be at least 2, if any at all */
	unsigned int num_teleporters;
/** Number of hot-areas */
	unsigned int num_areas;

/** Array of pointers to the platforms */
	Platform **platform;
/** Array of pointers to centers of gravity */
	CenterOfGravity **grav_center;
/** Array of pointers to the teleporters */
	Teleporter **teleporters;
/** Array of pointers ot the hot-areas */
	HotArea **areas;

/** Initial position of the taxi */
	int initial_x, initial_y;
/** initial fare */
	float base_fare;
/** stepping of fare, i.e. fare per unit of consumed fuel */
	float fare_step;
/** range of initial tip, passengers will start with */
	float min_base_tip, max_base_tip;
/** Number of successful deliveries needed to complete this level (i.e. deliveries with the
tip-counter being non-negative)*/
	int deliveries_to_go;
/** Whether an extra ship will be awarded upon completion of this level */
	bool extra_life;

/** The following deal with oversize levels. */
/** visible width and height of an oversized level */
	int visible_width, visible_height;
/** margin (both x and y) before scrolling takes place */
	int margin;
/** initial offsets of an oversized level */
	int initial_offset_x, initial_offset_y;
/** maximum x and y offsets for oversized levels (these are derived from the level's size and the visible size) */
	int max_offset_x, max_offset_y;
/** No descriptions */
	void config_changed();
  private:						// Private methods
/** Reads config for this level */
	void readLevel(const QDomElement & xdocElement);
/** option highlight_platforms */
	bool draw_platform_lines;
};

#endif
