/***************************************************************************
                          message.cpp  -  description
                             -------------------
    begin                : Wed Nov 28 2001
    copyright            : (C) 2001, 2002, 2007 by Thomas Friedrichsmeier
    email                : Thomas.Friedrichsmeier@ruhr-uni-bochum.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "defines.h"

#include "message.h"

#include <qcanvas.h>
#include <qstring.h>
#include <qrect.h>
#include <qobject.h>

#include "taxipilot.h"
#include "cdp.h"
#include "cachablepixmaparray.h"
#include "timemanager.h"

Message::Message() {
	phase = 4;			// which is the phase, in which the two text-sprites are not existent. See destructor
	mtm = new TimeManager (5, 10);
	mtm->pause ();
	ack_sprite = 0;
	message_sprite = 0;
	ack_pm = 0;
	message_pm = 0;
}

Message::~Message(){
	deleteSprites ();
	delete mtm;
}

void Message::deleteSprites () {
	if (phase != 4) {
		delete ack_sprite;
		delete message_sprite;
		CachablePixmapArray::discard (message_pm);
		CachablePixmapArray::discard (ack_pm);
		mtm->killTimers (this);
		phase = 4;
	}
}

void Message::displayMessage (QString & message, QString & ack_key, int below, int max_width) {
	kill ();

	int width;
	if (!max_width) {
		width = static_cast<int> (view->width () * 0.3);
	} else {
		width = max_width;
	}
	ack_sprite = new QCanvasSprite (ack_pm = cdp->createTextPixmapArray( i18n ( "Press '%1'...").arg(ack_key), SPACE_TEXT_FONT,
							Qt::SPACE_TEXT_COLOR, Qt::AlignHCenter | Qt::AlignTop,
							width, (int) (view->height () * 0.05), true, true, true), canvas);
	ack_sprite->setZ(MESSAGE_Z);

	if (!max_width) {
		width = static_cast<int> (view->width () * 0.8);
	}
	message_sprite = new QCanvasSprite (message_pm = cdp->createTextPixmapArray(message, MESSAGE_FONT,
							Qt::MESSAGE_COLOR, Qt::AlignHCenter | Qt::AlignBottom,
							width, (int) ((view->height () - below) * 0.4), true, true, true), canvas);
	message_sprite->setZ(MESSAGE_Z);

	QRect bound = message_sprite->boundingRect ();
/* Results where no better than without "optimization". Disabled
// Optimize Canvas-Chunksize
	old_chunk_size = canvas->chunkSize ();
	canvas->retune ( 16 + ((int) ((bound.width () + bound.height()) / 2) / 16) * 16 ); */

	text_dest_mid = (view->height() / 2) + view->contentsY () + below;
	text_dest_top = view->contentsY () - (bound.height() / 2);

	if (cdp->animate_messages ()) {
	// Move the texts to bottom center of view
		message_sprite->move((view->width() / 2) + view->contentsX (), (view->height() + (bound.height() / 2)) + view->contentsY ());
		message_sprite->show();

		bound = ack_sprite->boundingRect();
		ack_sprite->move((view->width() / 2) + view->contentsX (), (view->height() + (bound.height() / 2)) + view->contentsY ());

		message_speed = (- TEXT_SPEED/cdp->fps ());
		ack_speed = (- 2 * TEXT_SPEED/cdp->fps ());

		mtm->startTimer (this, cdp->framePeriod ());
		mtm->resume ();

		phase = 0;
	} else {
		message_sprite->move ((view->width() / 2) + view->contentsX (), text_dest_mid);
		message_sprite->show ();

		ack_sprite->move ((view->width() / 2) + view->contentsX (), text_dest_mid + 20);
		ack_sprite->show ();

		phase = 2; 	// not really necessary to set this, since timer not active

		canvas->update ();
	}
}

void Message::timerTick (int) {
	if (message_sprite->y () > text_dest_mid) {
		message_sprite->moveBy (0, message_speed);
	} else {
		if (!phase) {
			phase = 1;
			ack_sprite->show ();
		} else if (phase == 1) {
			ack_sprite->moveBy (0, ack_speed);
			if (ack_sprite->y() < (text_dest_mid + 20)) {
				phase = 2;
			}
		} else if (phase == 3) {
			message_sprite->moveBy (0, message_speed);
			ack_sprite->moveBy (0, ack_speed);
			if ((ack_sprite->y() < -20) && (message_sprite->y () < text_dest_top)) {
				deleteSprites ();
				tp->do_event_after_message ();
//				canvas->retune (old_chunk_size);
			}
		}
	}
	if (phase != 2) {
		canvas->update ();
	}
}

/** Message was acknowledged */
void Message::acknowledged (){
	if (phase < 3) {
		phase = 3;
		if (!cdp->animate_messages ()) {
			deleteSprites ();
			tp->do_event_after_message ();
		} else {
			ack_sprite->show ();	// might not have been shown yet
		}
	}
}

/** Kills any message that might still be acitve (called when game gets aborted) */
void Message::kill (){
	deleteSprites ();
//	canvas->retune (old_chunk_size);
}
