/***************************************************************************
                          splashscreen.cpp  -  description
                             -------------------
    begin                : Thu Dec 5 2002
    copyright            : (C) 2002 by Thomas Friedrichsmeier
    email                : tfry@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "splashscreen.h"

#include "cdp.h"
#include "message.h"
#include "taxipilot.h"
#include "timemanager.h"
#include "cachablepixmaparray.h"

#include <qcanvas.h>

SplashScreen::SplashScreen(const QString &config_file, bool show) : Screen () {
	init ();
	if (config_file) {
		QDomElement element = cdp->open_config_file(config_file);
		readScreen (element);
		readSplash (element);
	}
	if (show) SplashScreen::show ();
}

SplashScreen::SplashScreen(const QString background_file, const QString &foreground_file, int timeout, bool show) {
	init ();
	setForeground (foreground_file);
	setBackground (background_file);
	setTimeout (timeout);
	if (show) SplashScreen::show ();
}

void SplashScreen::init () {
	timeout_id = -1;
}

SplashScreen::~SplashScreen(){
	discard ();
}

void SplashScreen::readSplash (const QDomElement & docElement) {
}

void SplashScreen::setBackground (const QString & file) {
	if (file == bad_string_value) return;
	if (background) delete background;
	background = cdp->createQCPixmap(file, 640, 480, Serious);
}

void SplashScreen::setForeground (const QString & file) {
	if (file == bad_string_value) return;
	if (image) CachablePixmapArray::discard (image);
	image = CachablePixmapArray::create (file, 1, 640, 480);
}

void SplashScreen::setTimeout (int msecs) {
	if (msecs) {
		timeout_id = tm->startTimer (this, msecs);
	}
}

void SplashScreen::discard () {
	hide ();
}

void SplashScreen::show () {
	if (background) TPBase::canvas->setBackgroundPixmap (*background);
	if (image) {
		setSequence (image);
		Screen::show ();
	}
}

void SplashScreen::done () {
	tp->screenDone ();
}

void SplashScreen::timerTick (int id) {
/* if else */
	if (id == timeout_id) done ();
}
