/***************************************************************************
                          taxipilot.h  -  description
                             -------------------
    begin                : Don Mai 17 20:03:43 CEST 2001
    copyright            : (C) 2001, 2002, 2004, 2005 by Thomas Friedrichsmeier
    email                : Thomas.Friedrichsmeier@ruhr-uni-bochum.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TAXIPILOT_H
#define TAXIPILOT_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <qlcdnumber.h>
#include <qdial.h>
#include <qfileinfo.h>
#include <qcanvas.h>
#include <qwidget.h>
#include <qstring.h>
#include <qptrlist.h>
#include <qvaluelist.h>
#include <qstringlist.h>
#include <qdatetime.h>

#include <klocale.h>
//#include KAPP

#include "timedobject.h"
#include "tpbase.h"

class TaxipilotApp;
class Passenger;
class Taxi;
class Message;
class EQDial;
class AnimatedSprite;
class Credits;
class MoneyLCD;
class Highscores;
class QLabel;
class NotifyingLabel;
struct MissionDescription;

/** Taxipilot is the base class of the project */
class Taxipilot:public QWidget, public TimedObject, public TPBase {
	Q_OBJECT
public:
/** construtor */
	 Taxipilot(TaxipilotApp * parent = 0, const char *name = 0);
/** destructor */
	~Taxipilot();
	bool onActivePlatform();
/** This function really would be part of the constructor. It initializes Taxipilot by reading
the default mission, displaying the start-screen, and setting a few variables.
The thing is though, that it may also set the KAction gameNew to enabled or
disabled, which rests on this Action having been created in TaxiPIlotApp, which
in turn depends on having an instance of Taxipilot.
If you can think of something more elegant, please don't hesitate to tell me. */
	void init();
/** Takes care of any scrolling necessary (for oversized levels) */
	void doScroll();
/** In case several timers ran up: are we trying to scroll already? */
	bool scrolling;
/** our parent app. level needs to be able to access this */
	TaxipilotApp *app;
/** Should gear be toggled in the next chores? */
	bool toggle_gear;

	friend class TaxipilotApp;	/* Our beloved parent */
	friend class Platform;
	friend class Credits;

/** Passenger has boarded */
	void picked_up ();
/** No descriptions */
	void delivery_complete();
/* passenger is now ready for pick-up */
	void passenger_ready();
/** I really hate doing this, but I need to have a function that is called after a text
message was displayed, which will do, whatever needs to be done next.
Unfortunately, what needs to be done may vary. */
	void do_event_after_message();

/** Should be called, when the config has changed. Takes care of activating those
	changes. */
	void configChanged();

/** Called by a Screen (Level or Splash), when it is done */
	void screenDone ();

/** holds the meta-data for a mission */
	struct MissionDescription {
	/** The mission title */
		QString title;
	/** The author(s) */
		QString author;
	/** The abstract */
		QString abstract;
	/** Flase if the mission was not readable at all*/
		bool ok;
	/** Filename of splash image */
		QString splash_image;
	};

/** Reads in and returns the description of a mission. */
	MissionDescription getMissionDescription (const QString &file);

  private:
	Taxi *taxi;
	Passenger *passenger;
	Message *message_handler;
	Highscores *highscores;
	int start_level;

/** False if the game has not started yet. So we don't accidentally mess with undefinded variables */
	bool game_running;
/** Whether the game is currently paused (real pauses, splash-screens, messages)
 This should also be true, if no game is running at all. */
	bool game_paused;
/** pauses the game */
	void pauseGame(bool show_pause_message);

/** Whether the passengers have been created (needed in the destructor) */
	bool passengers_created;
/** Array of pointers to all passengers */

	int base_timer, passenger_timer, canvas_timer, init_timer;
	void do_chores();
/** Computes the gravity acting on the taxi and sets it. Also updates the speedDial */
	void do_gravity();
	int check_landed();			// returns platform landed on or -1
	void landing_func();		// this function is responsible for everything involved when landed
	void crash(int how);		// handle the crash
/** Reads in a mission-description file. */
	void readMission(const QString & file);

	struct SplashHandle {
	/** Filename of foreground-file ("" if no file) */
		QString foreground_file;
	/** Filename of background-file ("" if no file) */
		QString background_file;
	/** Message to be displayed ("" if no message) */
		QString message;
	/** Timeout, after which the splash disappears automatically */
		int timeout;
	};

	struct LevelHandle {
	/** Filename for this level */
		QString filename;
	/** Id of the taxi to be used in this level */
		QString taxi_id;
	/** Ids of the passengers to be used in this level */
		QStringList passenger_ids;
    /** Pointer to a SplashHandle (if this really is a SplashScreen, 0 if it is a real Level) */
		SplashHandle *splash;
	};

/** List of all levels in the mission */
	QValueList<LevelHandle> levels;

/** List of all splashes in the mission (only used for deleting the splashes!) */
	QPtrList<SplashHandle> splashes;

	struct TaxiHandle {
	/** Filename of the taxi */
		QString filename;
	/** Whether the taxi should appear on the start-screen */
		bool on_start_screen;
	/** Coordinates on startscreen */
		int x, y;
	};

/** List of pointers to taxis currently displayed on the start_screen */
	QPtrList<Taxi> taxis_on_start_screen;
	typedef QMap<QString, TaxiHandle> Taxis;
/** List of all taxis available in the mission (indexed by id, value is filename) */
	Taxis taxis;
/** id of the current taxi */
	QString current_taxi;

/** Filename of the current mission */
	QString current_mission;

/** Fuel capacity of taxis in this mission */
	float taxi_fuel_capacity;
/** Initial amount of fuel for the taxi(s) in this mission */
	float taxi_initial_fuel;

	struct PassengerHandle {
	/** filename, this passenger is to be constructed from */
		QString filename;
	/** Pointer to the (created) passenger */
		Passenger *pointer;
	/** Likelyhood of the passenger (100 is normal, lower numbers mean
		the passenger appears less often */
		int likelyhood;
	/** Whether the passenger appears on the start-screen */
		bool on_start_screen;
	/** The x_position of the passenger on the Startscreen */
		int x;
	/** The x_position of the passenger on the Startscreen */
		int y;
	};

/** A list of passengers currently visible on the start_screen */
	QPtrList<Passenger> passengers_on_start_screen;
/** A map (list) of all (created) passengers, indexed by their id */
	typedef QMap<QString, PassengerHandle> Passengers;
	Passengers passengers;

/** The filename of the startscreen itself */
	QString startscreen_file;
	QCanvasPixmap *start_screen;
/** number of current level (internal, i.e. starting with 0) */
	int current_level;
//	QString taxi_file;
	QLCDNumber *shipsLCD, *deliveries_LCD;		// Display stuff
	MoneyLCD *scoreLCD, *fareLCD, *tipLCD;
	EQDial *fuelDial, *speedDial;
	QLabel *statusLabel;
	NotifyingLabel *destinationLabel;
	float fare, tip;			// coming to be gaming stuff
	float tip_step;			// depends on passenger->impatience (avoids recalculation)
	float score;
	int ships, initial_ships;
/** This is needed, so a teleporter we land on does not beam us right away again */
	bool on_teleporter;
	int on_platform;			// which platform are we on, then?
	bool fueling;
	bool passenger_exists;
	bool passenger_onboard, passenger_waiting;	// internal gaming variables
	int active_platform;		// the platform_number of the next task (-1 if no task pending)
	QString string;				// just a universal string
/**  Saves the content of the status label when game gets paused */
	QString lastStatus;
/** Checks, whether everything is ok, en/disables Game->New accordingly and displays
	startscreen */
	void initMission ();

/** Generates and displays the current mission's startscreen */
	void display_start_screen();
/** Delete (the passengers on) the Startscreen */
	void delete_start_screen();

/** Creates all passengers */
	void createPassengers ();
/** Deletes all passengers (if they were created) */
	void deletePassengers ();
/** Selects a new passenger based on the provided categories */
	Passenger *selectPassenger (const QStringList &ids);

/** Switches to the taxi indicated by id */
	void useTaxi (const QString &id);
/** deletes the current taxi. Safe wrapper around "delete taxi" */
	void deleteTaxi ();

/** Switches to the specified level. If index==-1, the next level is used, otherwise the level specified by index. Previous level (if any) will be deleted.
	Aborts game (+ returns false) if critical errors occured, resumeGame()s otherwise */
	bool newLevel (int index=-1);

/** deletes the current screen. Safe wrapper around "delete taxi" */
	void deleteScreen ();

/** Displays a message as QCanvasText on the Canvas (scrolling it into view). */
	void display_message(int message_id);
/** Generates the credits text and graphics */
	void generate_credits();
/** Free all the memory occupied by the credits */
	void delete_credits();
/** used for fading the current level */
	QCanvasRectangle *fader;
/** for do_event_after_message */
	int event_after_message;
/** current offsets */
	int current_offset_x, current_offset_y;
/** Pointer to the credits (needed only to delete them) */
	Credits *credits;
/** title and author of current mission */
	QString mission_title, mission_author, mission_version;
/** Is there an active start-screen? (Needed in case of errors) */
	bool start_screen_active;
/** see TaxipilotApp->keyReleaseEvent () for documentation */
	int current_key_lr, current_key_ud;
/** An acceleration constant, that may abstract from refresh and recalculation rate */
	float acceleration;
  protected:					// Protected methods
	void timerTick(int id);
/** Pause the game when focus gets lost. */
	void focusOutEvent (QFocusEvent *e);
	public slots:				// Public slots
/** This function aborts the current game. Usually called by ask_abortGame, which
first asks, then calls this function */
	void abortGame();
	private slots:				// Private slots
/** Starts the game */
	void newGame();
/** Guess what! */
	void quitGame();
/** saves current game */
	void saveGame ();
/** load a saved game */
	void loadGame ();
/** Resumes a paused game */
	void resumeGame();
/** Pauses the game. */
	void pauseGame () { pauseGame (true); };
/** Selects a new Mission (i.e. a list of level- and passenger-description-files) */
	void selectMission();
/** Asks whether game should really be aborted, then calls abortGame () */
	void ask_abortGame();
/** show Highscores */
	void showHighscores ();
};

#endif
